local bn_assets = require("BattleNetwork.Assets")
local battle_helpers = require("Battle.Helpers")

local AUDIO = bn_assets.load_audio("break.ogg")
local ATTACK_TEXTURE = bn_assets.load_texture("spell_panel_shot.png")
local ATTACK_ANIM = bn_assets.fetch_animation_path("spell_panel_shot.animation")

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_IDLE")

	local field = actor:field()
	local tile_list = {}

	local query = function(ent)
		-- Any living entity blocks the spell.
		if Living.from(ent) ~= nil then return true end
		return false
	end

	for x = 0, 6, 1 do
		for y = 0, 6, 1 do
			local tile = field:tile_at(x, y)
			if tile and tile:is_walkable() and actor:is_team(tile:team()) and #tile:find_entities(query) == 0 then
				table.insert(tile_list, tile)
			end
		end
	end

	action:set_lockout(ActionLockout.new_sequence())

	local total_timer = (#tile_list * 8) + 8

	action.on_execute_func = function(self, user)
		local step1 = action:create_step()

		local cooldown = 0
		local index = 1

		battle_helpers.shuffle(tile_list)

		step1.on_update_func = function(self)
			if total_timer <= 0 then
				self:complete_step()
				return
			end

			if cooldown <= 0 and index <= #tile_list then
				cooldown = 8

				local tile = tile_list[index]
				index = index + 1

				local attack = create_attack(actor, props, tile)
				tile:set_state(TileState.Broken)

				field:spawn(attack, tile)

				local dust_fx = Poof.new()

				field:spawn(dust_fx, tile)

				Resources.play_audio(AUDIO)
			else
				cooldown = cooldown - 1
			end

			total_timer = total_timer - 1
		end
	end

	return action
end

function create_attack(user, props, tile)
	local spell = Spell.new(user:team())
	local direction = user:facing()
	spell:set_facing(direction)

	local spell_anim = spell:animation()

	local targets = user:field():find_tiles(function(found)
		if not user:is_team(found:team()) and not found:is_edge() then return true end
		return false
	end)

	battle_helpers.shuffle(targets)

	spell:set_texture(ATTACK_TEXTURE, true)

	spell:set_hit_props(
		HitProps.new(
			props.damage,
			props.hit_flags,
			props.element,
			props.secondary_element,
			user:context(),
			Drag.None
		)
	)

	spell_anim:load(ATTACK_ANIM)

	spell_anim:set_state("RED_TEAM")

	spell:set_offset(0, 12)

	spell_anim:apply(spell:sprite())

	spell_anim:set_playback(Playback.Loop)

	local dest = targets[1]

	spell.on_update_func = function(self)
		if self:deleted() then return end

		local current_tile = self:current_tile()

		if current_tile == dest and not self:is_sliding() then
			self:delete()
			return
		end

		current_tile:attack_entities(self)

		if current_tile:is_edge() then
			self:delete()
			return
		end

		if self:offset().y > 0 then
			self:set_offset(self:offset().x * 0.5, (self:offset().y - 4) * 0.5)
		end

		if not self:is_sliding() then
			self:slide(dest, 12)
		end
	end

	spell.on_collision_func = function(self, other) self:delete() end

	spell.on_attack_func = function(self, other) end

	spell.on_delete_func = function(self) self:erase() end

	spell.can_move_to_func = function(self, tile) return true end

	return spell
end
